clear all;
programmversion=('VibratoFinder | version: 1.00');
%========== E I N G A B E T E I L ================
% Pfad, in dem dieses Skript liegt:
programFolder=("C:/IhrOrdner/IhrUnterordner/VibratoFinder");
% Pfad, in dem zuerst nach zu oeffnenden Dateien gesucht werden soll
preferredFolder=("C:/IhrAnalysenOrdner/");
%===============================================================================
%                  Inhalt:                                                 Zeile
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% 1. Optionen...............................................................  20
% 2. Ordnerverzeichnisse erstellen..........................................  64
% 3. Datei laden............................................................  83
% 4. wenn noetig, Hertz in Cent umwandeln................................... 162
% 5. Tonhoehenkurve glaetten oder bereinigen................................ 192
% 6. nach potentiellen Sinusschwingungen im Vibratobereich suchen........... 328 
% 7. Vergleich Sinuswellen-Modell und Messdaten ueber Analysefenster........ 388
% 8. Ausgabe der Ergebnisse................................................. 579
%-------------------------------------------------------------------------------
%=============== O P T I O N E N ================
% Arbeitsmodi:
INTERAKTIV =false;% Standard ist true. Bei "false" keine Abfrage außer Dateiwahl
STAPELMODUS=false;% Standard ist false. Aktivieren mit "true"
%-------------------------------------------------
% Optionen fuer Ausgabe: wenn "true", werden folgende neue Dateien erstellt
AUSGABEPCO=false; % Centkurve, wenn Hz in Cent konvertiert werden musste
AUSGABEPC=false;  % geglätte Kurve, (nur nach Glaettung)
AUSGABEE=true;    % Ergebnisse (Zeitpunkt, Frequenz, Amplitude, Guete)
AUSGABES=true;    % Statistik (Zusammenfassung der Ergebnisse)
%-------------------------------------------------
% Optionen fuer Grafiken: 
ZeigeFreqAmp=true;% zeige Grafik: Frequenz + Auslenkung mit Kreisgroesse = Guete
KREISMIN =   4;   % minimale Kreisgroesse (fuer Punkte mit schlechter Guete)
KREISMAX =  20;   % maximale Kreisgruesse (fuer Punkte mit Guete = 100)
VORSCHAU = false; % Vorschaufenster fuer Sinusvergleich (true = VIEL langsamer!)
%-------------------------------------------------
% Optionen fuer Transformationen:
hertz2cent = 'ja';% Geladene Daten sind in Hertz. Alternative: 'nein'
glaettung = 1;    % keine Glaettung (1), Reduktion (2), Transposition (3)
%-------------------------------------------------
% allgemeine Optionen (sollten bei normaler Anwenung nicht geaendert werden):
STRENGE=2;        % Strenge des Kosinusvergleichs (1=leicht, 2=mittel, 3=schwer)
DSCHWELLE=50;     % Mindestguete* des Vibratis fuer Erfassung in Ergebnisliste
VSCHWELLE=90; 	  % Mindestguete* für Einbezug in statistische Auswertung
%                   *) Determinationskoeffizient bei positiven Korellationen
%                      zwischen Tonhoehenkurve und Sinusmodell
MINVIB=4;         % Untergrenze der zu messenden Vibrati (in Hz)
MAXVIB=10;        % Obergrenze  der zu messenden Vibrati (in Hz)
MAXAMB=700;       % Ambitus-Obergrenze (Max-Min) der zu messenden Vibrati (in Cent)
%-------------------------------------------------
% Optionen fuer Glaettung
mindestabstand=400;     % zulaessige Entfernung von Mediankurve (outlierDistance)
glaettungsfenster=50;   % Fenstergroesse (smoothingDistance)
fensterueberlappung=3/4;% Ueberlappung der Fenster
%-------------------------------------------------
%-------------------------------------------------
%=========== P R O G R A M M T E I L =============
%-------------------------------------------------
%-------------------------------------------------
%       Terminal-Ausgabe des Programmnamens
disp(programmversion); 
disp('-----------------------------'); 
graphics_toolkit='qt';                             % Oberfläche fuer Dialoge
%=================================================
%         2. Ordnerverzeichnisse erstellen
%--------------------------------------------------
disp('Verzeichnisse erstellen.............'); 
%programFolder=strcat(basicFolder,"/",programName);	% Verzeichnis des Programms
fileFolder=pwd;                                     % aktueller Ordner
path(path, programFolder); 					          % Programmverzeichnis zu 
%                                                     Octave hinzufuegen
if STAPELMODUS == true
  VibratoFinderStapel;                              % Stapelverarbeitung laden
  % Ausgabedatei: Kennwerte für Frequenz + Auslenkungen
  % Stapelnummer,n,f_Mittel,f_sd,f_fehler,f_min,f_q10,f_q25,f_median,f_q75,f_q90,f_max,
  %                a_Mittel,a_sd,a_fehler,a_min,a_q10,a_q25,a_median,a_q75,a_q90,a_max,
  Gesamtergebnis = zeros (nStapel,22);
  pfad=Dateipfad;
else
  nStapel=1;                                        % nur 1 Durchgang
endif
for iStapel=1:nStapel                               % fuer jede Datei
  %--------------------------------------------------
  %         3. Datei laden
  %--------------------------------------------------
  disp('Datei laden........................');
  if STAPELMODUS != true
    % Hinweisfenster
    response=helpdlg({'Bitte Tonhoehendatai waehlen.',
       'Es muss eine Textdatei sein,',
       'in der Zeitpunkte und Tonhoehen',
       'paarweise pro Zeile vorliegen.'});
    do 
      % Oeffnen-Dialog: Auswahl Dateiname und Pfad
      [dateiname, pfad, dateiindex]=uigetfile(preferredFolder,'oeffnen')
      % Dateiname ohne Endung (fuer spaetere Ausgaben):
      dateinamekurz=substr(dateiname,1,(rindex(dateiname,'.')-1)); 
      % vollstaendiger Dateipfad:
      datei=strcat(pfad,dateiname); 
      if exist(datei,'file')==2     % wenn Datei existiert,
        try                         % versuche,
          % (unbeweglicher) Fortschrittsbalken mit Feedback
          disp('Datei laden');
          X=load(datei);            % Datei einzulesen
        catch                       % wenn Einlesen nicht geht, Fehlermeldung:
          response=questdlg({'Datei entspricht nicht den inhaltlichen Anforderungen'
              'es duerfen keine Buchstaben oder Sonderzeichen enthalten sein'
              'Dezimalstellen muessen mit DezimalPUNKT getrennt werden'
              '(kein DezimalKomma'
              'es muessen zwei Spalten mit Zahlen vorliegen'
              'beide Spalten muessen gleich viele Zahlen enthalten.'
              'Bitte Datei ueberpruefen und neu laden'},
              'Fehler',
              'neue Datei waehlen',
              'Abbrechen',
              'neue Datei waehlen');
          if strcmp(response, 'Abbrechen')
            msgbox('Abgebrochen');
          endif;
        end_try_catch
      else                          % Datei existiert nicht; dann Hinweismeldung:
       response=questdlg('Datei konnte nicht gefunden werden'
           , 'Fehler', 'neue Datei waehlen','Abbrechen','neue Datei waehlen');
       if strcmp(response, 'Abbrechen')
          exit;     
       endif
      endif
      if exist('X','var')==1        % Wenn eingelesene Daten in Variable vorliegen
        weiter=logical(true);       % verlasse die Schleife
      else                          % Wenn eingelesene Daten nicht vorliegen
        weiter=logical(false);      % wiederhole die Dateiauswahl ab "do"
      endif
    until weiter==true
    to=X(:,1);  % to = Zeitwerte (original)
    po=X(:,2);  % po = pitch (Tonhoehenwerte)
    clear X;    % X kann jetzt geloescht werden
  else % STAPELMODUS == true
    dateiname=char(Dateiliste(iStapel,1));
    dateinamekurz=substr(dateiname,1,(rindex(dateiname,'.')-1)); 
    datei=strcat(Dateipfad,dateiname); 
    if exist(datei,'file')==2     % wenn Datei existiert,
     try                          % versuche,
       % (unbeweglicher) Fortschrittsbalken mit Feedback
       disp('Datei einlesen');
       disp(datei);
       X=load(datei);             % Datei einzulesen
       to=X(:,1);  % to= Zeitwerte (original)
       po=X(:,2);  % p = pitch (Tonhoehenwerte)
       clear X;    % X kann jetzt geloescht werden
      catch                       % wenn Einlesen nicht geht, Fehlermeldung:
       disp(datei)
       disp('Fehler beim Lesen der Datei');
       continue;
      end_try_catch
    else                          % Datei existiert nicht; dann Hinweismeldung:
      disp(datei)
      disp('Fehler beim Lesen der Datei');
      continue;
    endif
  endif % STAPELMODUS == true
  nto=length(to);          % Elemente in to (wird mehrfach gebraucht)
  %--------------------------------------------------
  %         4. wenn noetig, Hertz in Cent umwandeln
  %--------------------------------------------------
  if STAPELMODUS != true  && INTERAKTIV == true
    % wenn KEINE Stapelverarbtg + interaktiver Modus
    % Nutzer fragen, ob Tonhoehen in Hertz oder Cent stehen
    % (bei Lautstärkevibrato kann "Cent" angegeben werden, weil dann
    % die Daten so belassen bleiben, wie sie sind).
    hertz2cent = questdlg ({'Tonhoehenwerte muessen in Cent vorliegen.',
     'Sollen Hertz-Werte in Cent umgerechnet werden?.'},
     'Frage', 'ja', 'nein', 'ja');
  endif % STAPELMODUS != true
  if logical(strcmp(hertz2cent, 'ja')) % Wenn p in Hz ist
          disp('Cent aus Hz berechnen..............');
    % (unbeweglicher) Fortschrittsbalken mit Feedback
    pco=log2(po/440)*1200; % Konvertieren von Hz nach Cent (pco = pitchcent original)
  % todo: Ergebnisse speichern ([t,pco])
    if AUSGABEPCO==true
      disp('schreibe pco');
      ausgabedatei=strcat(pfad,dateinamekurz,'_pco.txt'); % vollstaendiger Dateipfad
      fid=fopen(ausgabedatei, 'w');            % zum Schreiben oeffnen
      fprintf(fid,'%% Cent-Kurve erstellt mit %s\r\n',programmversion); % Kopfzeile
      for it=1:nto
        fprintf(fid,'%.6f\t%.2f\r\n',to(it),pco(it));% schreibe Werte
      endfor
      fclose(fid);
    endif
  else                    % Wenn p bereits in Cent angegeben ist, 
    pco=po;               % dann soll pco von p uebernommen werden
  endif
  %--------------------------------------------------
  %         5. Tonhoehenkurve glaetten oder bereinigen
  %--------------------------------------------------
  zaehlertransHoch=0;   % Zaehler fuer nach oben transponierte Werte
  zaehlertransRunter=0; % Zaehler fuer nach unten transponierte Werte
  zaehlerReduktion=0;   % Zaehler fuer entfernte Werte
  if STAPELMODUS != true  && INTERAKTIV == true
  % wenn KEINE Stapelverarbtg + interaktiver Modus
  % Nutzerabfrage fuer Glaettung
    % Parameter fuer verschiedene Reduktions/Transpositionsmethoden
    glaettungsdialog = questdlg ({'Welche Glaettungsmethode?'},
       'Frage', 'Keine', 'Reduktion', 'Transposition', 'Keine');
    if strcmp(glaettungsdialog,'Keine')
      glaettung = 1;
    elseif strcmp(glaettungsdialog,'Reduktion')
      glaettung = 2;
    else
      glaettung = 3;
    endif
  endif
  switch (glaettung)
    case 1
      disp('keine Bereinigung');
    case 2
      disp('Reduktion');
    case 3
      disp('Transposition');
  endswitch
  % glaettung
  if glaettung != 1
    fortschritt = waitbar (0,'Schritt 1: Mediane und Varianzen berechnen');
    huerde=(200);                    % Schwelle fuer Bewegung Fortschrittsbalken
    % Matrix fuer Originale, Mediane, Varianzen und transponierte Werte:
    Glaettung=[pco,zeros(nto,3)];
    f1=ceil(glaettungsfenster/2);    % Fenster Abstand nach hinten
    f2=floor(glaettungsfenster/2);   % Fenster Abstand nach vorn
    n=nto-floor(glaettungsfenster/2);% Anzahl der folgenden Bearbeitungsschritte
     % Mediane und Varianzen erstellen
     for i=f1+1:n
      Glaettung(i,[2:3])=[median(pco((i-f1):(i+f2))),var(pco((i-f1):(i+f2)))];
      if i==huerde
        fortschritt = waitbar (i/n);% Fortschrittsbalken aktualisieren
        huerde=huerde+200;          % naechste Schwelle fuer Balkenbewegung
      endif
    endfor
    close(fortschritt);             % Fortschrittsbalken schliessen
    % Versatz, mit dem "nach vorn" und "nach hinten" gesucht wird
    ialt=round(glaettungsfenster*(1-fensterueberlappung)/2);  
    huerde=(200);                   % Schwelle fuer Bewegung Fortschrittsbalken
    % neuer Fortschrittsbalken
    fortschritt = waitbar (0,'Schritt 2: Mediane waehlen'); 
    for i=f1+1:n
      if Glaettung((i-ialt),3)<=Glaettung((i+ialt),3)% Wenn Varianz vorher kleiner
        Glaettung(i,1)=Glaettung((i-ialt),2);        % nimm Median von vorher
      else                                           % sonst
        Glaettung(i,1)=Glaettung((i+ialt),2);        % von nachher
      endif
      if i==huerde
        fortschritt = waitbar (i/n);% Fortschrittsbalken aktualisieren
        huerde=huerde+200;          % naechste Schwelle fuer Balkenbewegung
      endif
    endfor 
    close(fortschritt);             % Fortschrittsbalken schliessen
    huerde=200;
    if glaettung ==3                % Wenn Transposition gewuenscht:
      % Fortschrittsbalken neu
      fortschritt = waitbar (0,'Schritt 3: Ausreisser transponieren');
      for i=1:nto 
        if pco(i)-Glaettung(i,1)>mindestabstand  % Wenn Abstand zu Median zu hoch
          Glaettung(i,4)=pco(i)-1200;            % transponiere Wert nach unten
          zaehlertransRunter++;                  % zaehle diesen Vorfall
        elseif Glaettung(i,1)-pco(i)>mindestabstand % Wenn zu gering
          Glaettung(i,4)=pco(i)+1200;            % transponiere Wert nach oben
          zaehlertransHoch++;                    % zaehle diesen Vorfall
        else                                     % sonst aber
          Glaettung(i,4)=pco(i);                 % nimm den Wert, wie er ist
        endif
        if i==huerde
         fortschritt = waitbar (i/nto);% Fortschrittsbalken aktualisieren
         huerde=huerde+200;            % naechste Schwelle fuer Balkenbewegung
        endif
      endfor
      close(fortschritt);              % Fortschrittsbalken schliessen
      % Fortschrittsbalken neu (nach Transposition ist das Schritt 4)
      fortschritt = waitbar (0,'Schritt 4: Ausreisser reduzieren');
    elseif glaettung ==2            % Wenn Reduktion gewünscht
      Glaettung(:,4)=pco;           % nimm (erstmal) alle Werte, wie sie sind
      % Fortschrittsbalken neu (ohne Transposition ist das Schritt 3)
      fortschritt = waitbar (0,'Schritt 3: Ausreisser reduzieren');
    endif
    % Reduktion (wird auch nach Transposition durchgefuehrt
    nt=0;                           % Zaehler fuer neue Zeit und Tonhoehenwerte
    huerde=200;
    for i=1:nto 
        if Glaettung(i,4)-Glaettung(i,1)>mindestabstand% Abstd. zu Median zu gross
          zaehlerReduktion++;       % zaehle diesen Vorfall
          continue;                 % geh zum naechsten Element ueber
        elseif Glaettung(i,1)-Glaettung(i,4)>mindestabstand % Abstd. zu klein
          zaehlerReduktion++;       % zaehle diesen Vorfall
          continue;                 % geh zum nächsten Element ueber
        else                        % wenn Element innerhalb zulaessigr. Grenze
          nt++;                     % zaehle dieses Element
          t(nt,1)=to(i);            % fuege Zeitwert in neue Zeitreihe ein
          pc(nt,1)=Glaettung(i,4);  % fuege Centwert in neue Tonhoehenreie ein
        endif
        if i>=huerde
         fortschritt = waitbar (i/nto);% Fortschrittsbalken aktualisieren
         huerde=huerde+200;            % naechste Schwelle fuer Balkenbewegung
        endif
    endfor  
    close(fortschritt);       % Fortschrittsbalken schliessen
    % todo ergebnisse speichern [t,pc]
    if AUSGABEPC==true
      disp('schreibe pcg');
      ausgabedatei=strcat(pfad,dateinamekurz,'_pcg.txt');% vollstaendgr. Dateipfad
      fid=fopen(ausgabedatei, 'w');            % zum Schreiben oeffnen
      % Kopfzeile:
      fprintf(fid,'%% geglaettete Cent-Kurve erstellt mit %s\r\n',programmversion); 
      % schreibe Werte
      fprintf(fid,'%%%.f Punkte wurden durch Reduktion entfernt.\r\n',zaehlerReduktion);
      if glaettung==3
        % schreibe Werte
        fprintf(fid,'%%%.fPunkte wurden um 1200 Cent hochtransponiert\r\n',zaehlertransHoch);
        % schreibe Werte
        fprintf(fid,'%%%.fPunkte wurden um 1200 Cent runtertransponiert\r\n',zaehlertransRunter);
      endif
      for i=1:nt
        fprintf(fid,'%.6f\t%.2f\r\n',t(i),pc(i));% schreibe Werte
      endfor
      fclose(fid);
    endif
  else % glaettung != 1 => Wenn keine Reduktion/Transposition erwuenscht
    t=to;        % uebernimm alle Zeitwerte in neue Zeitreihe
    pc=pco;      % uebernimm alle Centwerte in neue Tonhoehenreihe
    nt=nto;      % uebernimm die Anzahl der Elemente fuer neue Reihe
  endif % glaettung != 1
  %--------------------------------------------------
  %         6. nach potentiellen Sinusschwingungen im Vibratobereich suchen
  %--------------------------------------------------
  % zur Erinnerung:
  % t  = alle Zeitpunkte 
  % pc = alle dazugehoerigen Tonhoehenwerte in Cent
  % nt = die Anzahl der Elemente in t und pc
  disp('gueltige lokale Minimae suchen.....');
  schwingungsdauerMax=1/MINVIB;            % laengste Schwingungsdauer
  schwingungsdauerMin=1/MAXVIB;            % kuerzeste Schwingungsdauer
  ndpc=nt-1;                               % Anzahl der Anstiege
  dpc=diff(pc);                            % Anstiege zwischen den Cent-Werten
  disp('Erste Ableitung nach Nulldurchgaegngen durchsuchen');
  lokalesMinimum=logical(zeros(ndpc,1));   % Vektor fuer alle negativen Anstiege
  nd2pc=ndpc-2;                            % Anzahl der Anstiege - 2*
  for i=2:nd2pc                            % fuer alle Anstiege ab Position 2*:
    if dpc(i)<=0 && dpc(i+1)>0             % Wenn positiv auf negativ folgt
      lokalesMinimum(i)=true;              % markiere die Stelle im Vektor mit 1
    endif                                  % sonst bleiben sie null
  endfor
  % *) -2, wegen Platz für Nachbarn von mins, die immer mit getestet werden. 
  %    es muss also immer noch ein Nachbarelement zur Verfügung stehen.
  disp('gefundene Nulldurchgaenge von negativ nach positiv:');
  disp(num2str(sum(lokalesMinimum)));
  % Berechnen der Zeitabstaende zwischen den Nulldurchgaengen (NDG)
  lokMinPosition=find(lokalesMinimum==true);% Positionen der NDG
  nMin=length(lokMinPosition);              % Anzahl der NDG 
  % Minimae und Alternativen ( fuer t und pc, jeweils min, min-1 und min+1)
  minAlt=zeros(3,4);
  % ausgewaehlte Alternativen
  tminAlt0=tminAlt1=zeros(2,1);
  % Minimae Dauern und Zeitpunkte zu nachfolgenden Minimae: 
  tminkorr=tmin=deltaMin=deltaMax=zeros(minAlt,1);
  % zur Erinnerung:
  % lokalesMinimum = Aussage fuer alle Elemente in pc und t, 
  %                  ob ein lokales Minimum vorliegt (1) oder nicht (0)
  % lokMinPosition = Nummer der Elemente in pc und t, an denen ein lokales Minimum
  %                  vorliegt
  % nMin           = Anzahl aller lokalen Minimae
  disp('Berechne Abstaende zwischen Nulldurchgaengen von - nach + in Ableitung');

  IMI=zeros(1,5);                % position in t, t, IMI in Sekunden, fmin, fmax
  nIMI=0;                        % Gesamtzahl der relevanten ima
  for i=2:nMin
    imi=t(lokMinPosition(i))-t(lokMinPosition(i-1)); % aktuelles ima
    if imi>schwingungsdauerMax      % wenn imi zu gross ist
      continue;                     % naechstes Intervall ueberpruefen
    elseif imi<schwingungsdauerMin  % wenn imi zu klein ist
      continue;                     % naechstes Intervall ueberpruefen
    else                            % sonst
      nIMI++;                       % 1 relevantes imi mehr
      IMI(nIMI,:)=[
        lokMinPosition(i-1),
        t(lokMinPosition(i-1)),
        imi,
        1/((t(lokMinPosition(i)-1)+t(lokMinPosition(i)))/2-(t(lokMinPosition(i-1)+1)+t(lokMinPosition(i-1)))/2),
        1/((t(lokMinPosition(i)+1)+t(lokMinPosition(i)))/2-(t(lokMinPosition(i-1)-1)+t(lokMinPosition(i-1)))/2)
        ];  % trage Startzeit und Dauer des ima ein
    endif
  endfor
  %--------------------------------------------------
  %         7. Vergleich Sinuswellen-Modell und Messdaten ueber Analysefenster
  %--------------------------------------------------
  disp('Abgleich Sinuswellen/Messdaten.....');
  % Ergebnisse = 3-Spalten-Matrix fuer alle potentiellen Einzelschwingungen: 
  % 1. Zeitpunkt, 2. Frequenz 3. Spannweite (doppelte Amplitude)
  Ergebnisse=zeros(nIMI,3); 
  guete = zeros(nIMI,1, 'int8'); % 4. Guete (Determinationskoeffizient)
  fdif = zeros(nIMI,1); % 4. zum Ueberpruefen der differenz nach Iteration (nur intern, kann am Ende Raus !!!
  % Fortschrittsbalken
  fortschritt = waitbar (0,'Abgleich der Sinuswellen mit Messdaten');       
  huerde=int8(25);                     % Schwelle fuer Bewegung Fortschrittsbalken
  pi2=2*pi;                            % haufig gebrauchte Konstante
  if VORSCHAU==true
    figure (1)
  endif
  for i=1:nIMI                         % fuer alle potentiellen Einzelschwingungen
    if i==huerde
      fortschritt = waitbar (i/nIMI);  % Fortschrittsbalken aktualisieren
      huerde=huerde+25;                % naechste Schwelle fuer Balkenbewegung
    endif
    % Ausgehend vom lokalen Minimum der Schwingung wird ein Analysefenster 
    % mit der 1.5/2.5/3.5 -fachen aktuellen Schwingungsdauer aufgespannt,
    % in dessen Mitte das lolale Minimum liegt.
    fenstermitte=IMI(i,2);              % Zeitpunkt des Minimums = Fenstermitte
    if STRENGE == 2
      halbfenster=IMI(i,3)*1.25;        % Halbe Analysefenstergroesse
    elseif STRENGE == 1
      halbfenster=IMI(i,3)*0.75;        % Halbe Analysefenstergroesse
    else
      halbfenster=IMI(i,3)*1.75;        % Halbe Analysefenstergroesse
    endif
    fensterstart=min(find(t>=(fenstermitte-halbfenster)));
    fensterende=max(find(t<=(fenstermitte+halbfenster)));
    % ABBRUCHBEDINGUNG 1: AMBITUS 
    pcspanne=range(pc(fensterstart:fensterende)); % Ambitus im Fenster
    if pcspanne>MAXAMB
      %disp('Analysefenster uebersprungen: Ambitus zu gross');
      printf('skip_A.');
      continue;                        % weiter mit naechstem Element
    endif
    % ABBRUCHBEDINGUNG 2: Anzahl der Elemente muss mindestens 75 auf 100ms sein
    fenstergroesse=t(fensterende)-t(fensterstart);  % Analysefensters in Sekunden
    nfenster=length(t(fensterstart:fensterende));   % Elemente im Analysefenster
    if nfenster<fenstergroesse*50 
      %disp('Analysefenster uebersprungen: zu wenig Elemente');
      printf('skip_E.');
      continue;                        % weiter mit naechstem Element
    endif 
    hkorr=IMI(i,2)-t(fensterstart);        % korrigiertes Halbfenster
    % Zeitwerte im Analysefenster:
    tfenster=t(fensterstart:fensterende)-t(fensterstart); 
    % Tonhoehenwerte im Zeitfenster skaliert zwischen -1 und 1:
    pcfensterMin=min(pc(fensterstart:fensterende)); % minimale Tonhoehe
    pfenster=2*(pc(fensterstart:fensterende)-pcfensterMin)/pcspanne-1;
    % Ambitus einer Sinusschwingung ueber das Zeitfenster
    % todo: F = alle Frequenzen
    % Berechnung der korrigierten Nullstelle aus den drei Messpunkten 
    % des aktuellen Minimums (ueber quadratische Kurve -> gaussscher Algorithm.)
    %     1  t1  t1^2  pc1
    % A = 1  t2  t2^2  pc2  % Ausgangsmatrix
    %     1  t3  t3^2  pc3
    h=find(tfenster==hkorr)+1;
    A=[ones(3,1),tfenster((h-1):(h+1)),tfenster((h-1):(h+1)).^2,pfenster((h-1):(h+1))]; % Ausgangsmatrix
    A2(1,:)=-1*(A(2,1)/A(1,1)).*A(1,:)+A(2,:);            % Erste Berechnung
    A2(2,:)=-1*(A(3,1)/A(1,1)).*A(1,:)+A(3,:);            % siehe Mathebuch ...
    A3=-1*(A2(2,2)/A2(1,2)).*A2(1,:)+A2(2,:);
    x3=A3(4)/A3(3);                  % berechneter Wert fuer a
    x2=(A2(1,4)-x3*A2(1,3))/A2(1,2); % berechnet den Wert fuer b
    % x1=A(1,4)-x2*A(1,2)-x3*A(1,3); % berechnet den Wert fuer c, 
    %                                  c brauchen wir aber hier gar nicht mehr
    % Nullstelle: Ableitung: 2*ax+b=0   ax=-b/2     x=-b/(2*a)
    tMinKor=-x2/(2*x3); % berechnete Nullstelle = korrigierte Nullstelle
    % Alternativen finden
    fopt=1/IMI(i,3);                 % korrigierte Nullstelle
    b=pi2*fopt;                      % b in cos(bt-c) fuer das Kosinusmodell
    c=pi+b*tMinKor;                  % c in cos(bt-c) fuer das Kosinusmodell
    modell=cos(b.*tfenster-c);       % Kosinusmodell
    % Hanning-Fenster zum Glaetten der Raender erstellen
    hanfenster=0.5-(0.5*cos(2*pi*tfenster/fenstergroesse));
    % Faltung mit Hanning-Fenster und Korrelation von Messwert und Modell
    corrfenster=corr((hanfenster.*[modell,pfenster]))(2,1);
    % ABBRUCHBEDUNGUNG KORRELATION (GUETE)
    if corrfenster<0.75*sqrt(DSCHWELLE/100)
      continue;
    endif 
    % berechne Alternativen mit binaerer suche
    fmin=IMI(i,5);                         % zwischen kleinstmoeglicher
    fmax=IMI(i,4);                         % und groesstmoeglicher Frequenz
    startdistanz=max(fopt-fmin,fmax-fopt); % Anfangsdistanz
    fbest=fopt;                            % derzeit ist fopt die beste frequenz
    corrbest=corrfenster;                  % derzeit die beste guete
    while startdistanz >= 0.05       % binaere Suche, bis fmax-fmin < 0.05 Hz ist
      if startdistanz>1                    % da Messpunkte fehlen koennten, darf
        startdistanz=1;                    % startdistanz nicht zu gross werden
      endif
      F=[fbest-startdistanz,fbest+startdistanz]; % Vergleichs-(Nachbar-)frequenzen
      B=pi2.*F;                            % alle b Werte
      C=pi+B.*tMinKor;                     % alle c Werte
      MODELL=cos(B.*tfenster-C);           % alle Modelle
      CORRFENSTER=zeros(2,3);              % Matrix fuer Frequenz und Korrelation
      CORRFENSTER(1,1)=corr((hanfenster.*[MODELL(:,1),pfenster]))(2,1);
      CORRFENSTER(1,2)=corrbest;
      CORRFENSTER(1,3)=corr((hanfenster.*[MODELL(:,2),pfenster]))(2,1);
      CORRFENSTER(2,1)=fbest-startdistanz;
      CORRFENSTER(2,2)=fbest;
      CORRFENSTER(2,3)=fbest+startdistanz;    
      % neue Entscheidung uber beste Frequenz
      best=find(CORRFENSTER(1,:)==max(CORRFENSTER(1,:))); % groesste Korrelation
      corrbest=CORRFENSTER(1,best);
      fbest=CORRFENSTER(2,best);           % Frequenz mit groesster Korrelation
      startdistanz=startdistanz/2;         % Verkleinern der Schritte zur Neusuche
    endwhile
      Ergebnisse(i,:)=[IMI(i,2),fbest,pcspanne];         % Ergebnisliste
      guete(i)=int8(round((abs(corrbest)*corrbest)*100));% Liste mit Guete
      % fdif(i)=fbest-fopt;         % Abweichung von Ursprungshypothese
                                    % nur zur Evaluation noetig
    %-------------- Vorschau -------------
    if VORSCHAU==true
      plot(tfenster,hanfenster.*MODELL,tfenster,hanfenster.*pfenster);
      axis ([0, 0.5, -1, 1]);
      axis ([0, 0.5, -1, 1]);
      pause(0.05);
    endif
  endfor
  if VORSCHAU==true
    close(1)
  endif
  close(fortschritt);
  % plot(fdif,Ergebnisse(:,2),'.'); % Abweichung von Ursprungshypothese
                                    % nur zur Evaluation noetig
  % Elemente in Ergebnisliste mit schlechter Korrelation rausschmeissen
  disp('Ergebnisliste bereinigen');
  nE=nIMI-(length(find(guete<=DSCHWELLE))); % Anzahl sauberer Ergebnisse
  Ergebnis=zeros(nE,4);                     % saubere Ergebnisliste
  j=1;
  for i=1:nIMI                            % fuer alle Ergebnisse
    if guete(i)<=DSCHWELLE                % wenn Korrelation <= Guetekriterium
      continue;                           % ueberspringen (da schlecht)
    endif
    Ergebnis(j,:)=single([Ergebnisse(i,1),Ergebnisse(i,2),Ergebnisse(i,3)/2,single(guete(i))]);        % sonst als hinreichend uebernehmen und Ambitus in Amplitude umrechnen
    j++;
  endfor
  %clear Ergebnisse;                       % unsaubere Ergebnisse loeschen
  if ZeigeFreqAmp==true
    % Darstellung der Ergebnisse
    disp('Ergebnisse anzeigen');
    % plotte den Punkt zweidimensional auf den Achsen Frequenz und Amplitude
    % die Punktgroesse ist abhaengig von der Korrelation (der Guete)
    % der Anpassung. Je groesser der Punkt, desto besser die Korellation
    msizes=int8(round((KREISMAX-KREISMIN)*(Ergebnis(:,4)-DSCHWELLE)/(100-DSCHWELLE))+KREISMIN);% Guete 
    figure (2)                              % neue Grafik
    hold on                                 % Grafik beibehalten (Werte hinzufuegen)
    for i=1:nE                              % fuer alle sauberen Punkte
      plot(Ergebnis(i,2),Ergebnis(i,3),'ok','markersize',msizes(i));
      pause(0.02);
    endfor
    hold off
    axis ([MINVIB, MAXVIB, 0, MAXAMB/2]);
    xlabel('Frequenz in Hz');
    ylabel('Auslenkung in Cent');
    title(dateiname);
    saveas(2,strcat(pfad,dateinamekurz),'pdf');
    close (2);
  endif
	auswahl=find(Ergebnis(:,4)>=VSCHWELLE); % Selektion der besten Guete
	freqStat=Ergebnis(auswahl,2);           % Frequenzen der besten Vibrati
	ampStat=Ergebnis(auswahl,3);            % Amplituden der besten Vibrati
	% Stapelnummer,n,f_Mittel,f_sd,f_min,f_q10,f_q25,f_median,f_q75,f_q90,f_max,a_Mittel,a_sd,a_min,a_q10,a_q25,a_median,a_q75,a_q90,a_max,
	Gesamtergebnis(iStapel, 1) = iStapel;
	Gesamtergebnis(iStapel, 2) = length(auswahl);
	Gesamtergebnis(iStapel, 3) = mean(freqStat);
	Gesamtergebnis(iStapel, 4) = std(freqStat);
	Gesamtergebnis(iStapel, 5) = Gesamtergebnis(iStapel, 4)/sqrt(Gesamtergebnis(iStapel, 2));
	Gesamtergebnis(iStapel, 6) = min(freqStat);
	Gesamtergebnis(iStapel, 7) = quantile(freqStat,0.1);
	Gesamtergebnis(iStapel, 8) = quantile(freqStat,0.25);
	Gesamtergebnis(iStapel, 9) = median(freqStat);
	Gesamtergebnis(iStapel,10) = quantile(freqStat,0.75);
	Gesamtergebnis(iStapel,11) = quantile(freqStat,0.9);
	Gesamtergebnis(iStapel,12) = max(freqStat);
	Gesamtergebnis(iStapel,13) = mean(ampStat);
	Gesamtergebnis(iStapel,14) = std(ampStat);
	Gesamtergebnis(iStapel,15) = Gesamtergebnis(iStapel, 14)/sqrt(Gesamtergebnis(iStapel, 2));
	Gesamtergebnis(iStapel,16) = min(ampStat);
	Gesamtergebnis(iStapel,17) = quantile(ampStat,0.1);
	Gesamtergebnis(iStapel,18) = quantile(ampStat,0.25);
	Gesamtergebnis(iStapel,19) = median(ampStat);
	Gesamtergebnis(iStapel,20) = quantile(ampStat,0.75);
	Gesamtergebnis(iStapel,21) = quantile(ampStat,0.9);
	Gesamtergebnis(iStapel,22) = max(ampStat);
  %--------------------------------------------------
  %         8. Ausgabe der Ergebnisse
  %--------------------------------------------------
  disp('Ausgabe der Ergebnisse.............');
  auswahl=find(Ergebnis(:,4)>=VSCHWELLE); % Selektion der besten Guete
  freqStat=Ergebnis(auswahl,2);           % Frequenzen der besten Vibrati
  ampStat=Ergebnis(auswahl,3);            % Amplituden der besten Vibrati
  freqStat2=Ergebnis(:,2);                % Frequenzen aller Vibrati
  ampStat2=Ergebnis(:,3);                 % Amplituden aller Vibrati
  if STAPELMODUS != true
    %Auswertung:
    sE=sprintf('Es wurden %.f Schwingungen mit D >= %.0f gefunden.\n', length(auswahl), VSCHWELLE);
    sE=strcat(sE,sprintf('Gesamtdauer aller Vibrati: %.1f Sekunden\n',sum(1./Ergebnis(auswahl,2))));
    sE=strcat(sE,sprintf('\n'));
    sE=strcat(sE,sprintf('Analyse der Vibratofrequenzen: \n'));
    sE=strcat(sE,sprintf('arithm. Mittel:\t\t%.1f Hz\n',mean(freqStat)));
    sE=strcat(sE,sprintf('Standardfehler Mittel:\t%.3f Hz\n',std(freqStat)/sqrt(length(auswahl))));    
    sE=strcat(sE,sprintf('Median:\t\t\t%.1f Hz\n',median(freqStat)));
    sE=strcat(sE,sprintf('Spannweite:\t\t%.1f Hz (von %.1f Hz-%.1f Hz)\n',range(freqStat),min(freqStat),max(freqStat)));
    sE=strcat(sE,sprintf('Interdezilabstand:\t\t%.1f Hz (von %.1f Hz-%.1f Hz)\n', quantile(freqStat,0.9)-quantile(freqStat,0.1), quantile(freqStat,0.1),quantile(freqStat,0.9)));
    sE=strcat(sE,sprintf('Interquartilsabstand:\t%.1f Hz (von %.1f Hz-%.1f Hz)\n', quantile(freqStat,0.75)-quantile(freqStat,0.25), quantile(freqStat,0.25),quantile(freqStat,0.75)));
    sE=strcat(sE,sprintf('Standardabweichung:\t%.1f Hz\n',std(freqStat)));
    sE=strcat(sE,sprintf('\n'));
    sE=strcat(sE,sprintf('Analyse der Vibratoamplituden: \n'));
    sE=strcat(sE,sprintf('arithm. Mittel:\t\t%.f Cent\n',mean(ampStat)));
    sE=strcat(sE,sprintf('Standardfehler Mittel:\t%.3f Hz\n',std(ampStat)/sqrt(length(auswahl))));    
    sE=strcat(sE,sprintf('Median:\t\t\t%.f Cent\n',median(ampStat)));
    sE=strcat(sE,sprintf('Spannweite:\t\t%.f Cent (von %.1f Cent-%.1f Cent)\n',range(ampStat),min(ampStat),max(ampStat)));
    sE=strcat(sE,sprintf('Interdezilabstand:\t\t%.f Cent (von %.1f Cent-%.1f Cent)\n', quantile(ampStat,0.9)-quantile(ampStat,0.1), quantile(ampStat,0.1),quantile(ampStat,0.9)));
    sE=strcat(sE,sprintf('Interquartilsabstand:\t%.f Cent (von %.1f Cent-%.1f Cent)\n', quantile(ampStat,0.75)-quantile(ampStat,0.25), quantile(ampStat,0.25),quantile(ampStat,0.75)));
    sE=strcat(sE,sprintf('Standardabweichung:\t%.f Cent\n',std(ampStat)));
    sE=strcat(sE,sprintf('\n'));
    sE=strcat(sE,sprintf('\n'));
    sE=strcat(sE,sprintf('Insgesamt wurden %.f Schwingungen mit D >= %.0f gefunden.\n', length(Ergebnis(:,1)), DSCHWELLE));
    sE=strcat(sE,sprintf('Gesamtdauer aller Vibrati: %.1f Sekunden\n',sum(1./Ergebnis(:,2))));
    sE=strcat(sE,sprintf('\n'));
    sE=strcat(sE,sprintf('Analyse der Vibratofrequenzen: \n'));
    sE=strcat(sE,sprintf('arithm. Mittel:\t\t%.1f Hz\n',mean(freqStat2)));
    sE=strcat(sE,sprintf('Standardfehler Mittel:\t%.3f Hz\n',std(freqStat2)/sqrt(length(Ergebnis(:,1)))));        
    sE=strcat(sE,sprintf('Median:\t\t\t%.1f Hz\n',median(freqStat2)));
    sE=strcat(sE,sprintf('Spannweite:\t\t%.1f Hz (von %.1f Hz-%.1f Hz)\n',range(freqStat2),min(freqStat2),max(freqStat2)));
    sE=strcat(sE,sprintf('Interdezilabstand:\t\t%.1f Hz (von %.1f Hz-%.1f Hz)\n', quantile(freqStat2,0.9)-quantile(freqStat2,0.1), quantile(freqStat2,0.1),quantile(freqStat2,0.9)));
    sE=strcat(sE,sprintf('Interquartilsabstand:\t%.1f Hz (von %.1f Hz-%.1f Hz)\n', quantile(freqStat2,0.75)-quantile(freqStat2,0.25), quantile(freqStat2,0.25),quantile(freqStat2,0.75)));
    sE=strcat(sE,sprintf('Standardabweichung:\t%.1f Hz\n',std(freqStat2)));
    sE=strcat(sE,sprintf('\n'));
    sE=strcat(sE,sprintf('Analyse der Vibratoamplituden: \n'));
    sE=strcat(sE,sprintf('arithm. Mittel:\t\t%.f Cent\n',mean(ampStat2)));
    sE=strcat(sE,sprintf('Standardfehler Mittel:\t%.3f Hz\n',std(ampStat2)/sqrt(length(Ergebnis(:,1)))));        
    sE=strcat(sE,sprintf('Median:\t\t\t%.f Cent\n',median(ampStat2)));
    sE=strcat(sE,sprintf('Spannweite:\t\t%.f Cent (von %.1f Cent-%.1f Cent)\n',range(ampStat2),min(ampStat2),max(ampStat2)));
    sE=strcat(sE,sprintf('Interdezilabstand:\t\t%.f Cent (von %.1f Cent-%.1f Cent)\n', quantile(ampStat2,0.9)-quantile(ampStat2,0.1), quantile(ampStat2,0.1),quantile(ampStat2,0.9)));
    sE=strcat(sE,sprintf('Interquartilsabstand:\t%.f Cent (von %.1f Cent-%.1f Cent)\n', quantile(ampStat2,0.75)-quantile(ampStat2,0.25), quantile(ampStat2,0.25),quantile(ampStat2,0.75)));
    sE=strcat(sE,sprintf('Standardabweichung:\t%.f Cent\n',std(ampStat2)));
    a=msgbox(sE, 'Ergebnis'); % Anzeige der Ergebnisse
  endif 
  % Statistik speichern
  if AUSGABES==true
    disp('schreibe Statistik');
    ausgabedatei=strcat(pfad,dateinamekurz,'_statistik.txt'); % vollstaendiger Dateipfad
    fid=fopen(ausgabedatei, 'w');            % zum Schreiben oeffnen
    fprintf(fid,'%% Statistik fuer %s erstellt mit %s\r\n',dateiname,programmversion);
    fprintf(fid,'n\t%.f\r\n',length(auswahl));
    fprintf(fid,'D\t%.f\r\n',VSCHWELLE);
    fprintf(fid,'Gesamtdauer in Sekunden:\t%.1f\r\n',sum(1./Ergebnis(auswahl,2)));
    fprintf(fid,'arithm. Mittel in Hz:\t%.1f\r\n',mean(freqStat));
    fprintf(fid,'Standardfehler Mittel:\t%.3f\n',std(freqStat)/sqrt(length(auswahl)));    	 
    fprintf(fid,'Median in Hz:\t%.1f\r\n',median(freqStat));
    fprintf(fid,'Spannweite in Hz:\t%.1f\r\n',range(freqStat));
    fprintf(fid,'Interdezilabstand in Hz:\t%.1f\r\n', quantile(freqStat,0.9)-quantile(freqStat,0.1));
    fprintf(fid,'Interquartilsabstand in Hz:\t%.1f\r\n', quantile(freqStat,0.75)-quantile(freqStat,0.25));
    fprintf(fid,'Standardabweichung in Hz:\t%.1f\n',std(freqStat));
    fprintf(fid,'arithm. Mittel in Cent:\t%.1f\r\n',mean(ampStat));
	  fprintf(fid,'Standardfehler Mittel in Cent:\t%.3f\n',std(ampStat)/sqrt(length(auswahl))); 
    fprintf(fid,'Median in Cent:\t%.1f\r\n',median(ampStat));
    fprintf(fid,'Spannweite in Cent:\t%.1f\r\n',range(ampStat));
    fprintf(fid,'Interdezilabstand in Cent:\t%.1f\r\n', quantile(ampStat,0.9)-quantile(ampStat,0.1));
    fprintf(fid,'Interquartilsabstand in Cent:\t%.1f\r\n', quantile(ampStat,0.75)-quantile(ampStat,0.25));
    fprintf(fid,'Standardabweichung in Cent:\t%.1f\n',std(ampStat));
	% Stapelnummer,n,f_Mittel,f_sd,f_min,f_q10,f_q25,f_median,f_q75,f_q90,f_max,a_Mittel,a_sd,a_min,a_q10,a_q25,a_median,a_q75,a_q90,a_max,
    fprintf(fid,'--------------------------------------------\r\n');
    fprintf(fid,'Uebersicht:\r\n');
    fprintf(fid,'%%Dateiname\tn\tf_mittel\tf_sd\tf_se\tf_min\tf_q10\tf_q25\tf_median\tf_q75\tf_q90\tf_max\ta_mittel\ta_sd\ta_se\ta_min\ta_q10\ta_q25\ta_median\ta_q75\ta_q90\ta_max\r\n');
  	fprintf(fid,'%s\t%.f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.1f\t%.1f\t%.3f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\r\n',char(dateinamekurz),Gesamtergebnis(iStapel,[2:22]));
	  fclose(fid);
  endif
  % Ergebnisse speichern
  if AUSGABEE==true
    disp('schreibe Ergebnisse');
    ausgabedatei=strcat(pfad,dateinamekurz,'_ergebnisse.txt'); % vollstaendiger Dateipfad
    fid=fopen(ausgabedatei, 'w');            % zum Schreiben oeffnen
    fprintf(fid,'%%t\tf in Hz\ta in Cent\tD\r\n');
    for i=1:nE
      fprintf(fid,'%.6f\t%.2f\t%.1f\t%.f\r\n',Ergebnis(i,:));
    endfor
    fclose(fid);
  endif
endfor % iStapel=1:nStapel
if STAPELMODUS == true
  disp('schreibe Gesamtergebnis');
  ausgabedatei=strcat(Dateipfad,dateinamekurz,'_Gesamtergebnis.txt'); % vollstaendiger Dateipfad
  fid=fopen(ausgabedatei, 'w');            % zum Schreiben oeffnen
  % Stapelnummer,n,f_Mittel,f_sd,f_min,f_q10,f_q25,f_median,f_q75,f_q90,f_max,a_Mittel,a_sd,a_min,a_q10,a_q25,a_median,a_q75,a_q90,a_max,
  fprintf(fid,'%%Dateiname\tn\tf_mittel\tf_sd\tf_se\tf_min\tf_q10\tf_q25\tf_median\tf_q75\tf_q90\tf_max\ta_mittel\ta_sd\ta_se\ta_min\ta_q10\ta_q25\ta_median\ta_q75\ta_q90\ta_max\r\n');
  for i=1:nStapel
    fprintf(fid,'%s\t%.f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.1f\t%.1f\t%.3f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\r\n',char(Dateiliste(i,1)),Gesamtergebnis(i,[2:22]));
  endfor
  fclose(fid);
endif
msgbox('fertig.');
%=================================================
%     Terminal-Ausgabe zum Ende des Programms
       disp('-----------------------------'); 
       disp(' Zum Beenden "exit" eingeben');
       disp('-----------------------------'); 
